package flink.zanxiangnet.ad.monitoring.stream;

import com.google.common.collect.Lists;
import flink.zanxiangnet.ad.monitoring.trigger.TimerCountTrigger;
import org.apache.flink.streaming.api.datastream.KeyedStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.functions.windowing.WindowFunction;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;

import java.util.List;

public class KeyedBatchStream<T, KEY> {

    private final KeyedStream<T, KEY> stream;
    // 缓存刷新的间隔时间
    private final Time bufferRefreshTime;
    // 缓存的最大数据量
    private final Long maxBufferCount;

    public KeyedBatchStream(KeyedStream<T, KEY> stream, Long maxBufferCount, Time bufferRefreshTime) {
        this.stream = stream;
        this.bufferRefreshTime = bufferRefreshTime;
        this.maxBufferCount = maxBufferCount;
    }

    public SingleOutputStreamOperator<List<T>> toBatch() {
        return stream.window(TumblingEventTimeWindows.of(bufferRefreshTime))
                .trigger(new TimerCountTrigger<>(maxBufferCount))
                // 这里不能使用 lambda表达式，flink无法推测类型
                .apply(new WindowFunction<T, List<T>, KEY, TimeWindow>() {
                    @Override
                    public void apply(KEY key, TimeWindow window, Iterable<T> iterable, Collector<List<T>> collector) throws Exception {
                        collector.collect(Lists.newArrayList(iterable));
                    }
                });
    }
}
